
#include <Poco/Net/Socket.h>
#include <Poco/Net/StreamSocket.h>
#include <Poco/Exception.h>
#include <Poco/Net/NetException.h>
#include <Poco/Logger.h>
#include <Poco/Format.h>
#include <Poco/Timespan.h>
#include "Dispatcher.hpp"
#include "Handler.hpp"


using Poco::Logger;
using Poco::NoThreadAvailableException;
using Poco::Net::ConnectionResetException;
using Poco::Net::Socket;
using Poco::Net::StreamSocket;
using Poco::Timespan;


Dispatcher::Dispatcher()
{
}


void Dispatcher::run()
{
	Logger& log = Logger::get("server");
	const Timespan TIMEOUT(0, 100000);

	while (true)
	{
		map<StreamSocket, Message*> messages;
		dequeue(messages);
		for (map<StreamSocket, Message*>::iterator it = messages.begin(); it != messages.end(); ++it)
		{
			if (it->first.poll(TIMEOUT, Socket::SELECT_READ))
			{
				try
				{
					Message* msg = messages[it->first];
					Handler* hnd = new Handler(msg);
					// if no thread is available, then put into queue and try later
					try
					{
						log.debug("Dispatcher::run(): starting processor for session " + Poco::format("%ld", msg->session()->id()));
						ThreadPool::defaultPool().start(*hnd);
					}
					catch (NoThreadAvailableException& exc)
					{
						log.debug("Dispatcher::run(): cannot start processor for session " + Poco::format("%ld", msg->session()->id()));
						enqueue(msg);
						delete hnd;
					}
				}
				catch (ConnectionResetException& exc)
				{
					log.debug("Dispatcher::run(): connection reset");
				}
			}
			else
			{
				log.debug("Dispatcher::run(): socket not readable, enqueuing it");
				enqueue(it->second);
			}
		}
	}
}


void Dispatcher::enqueue(Message* message)
{
	FastMutex::ScopedLock lock(_mutexQueue);
	_queue.push_back(message);
	_condQueue.signal();
}


void Dispatcher::dequeue(map<StreamSocket, Message*>& messages)
{
	FastMutex::ScopedLock lock(_mutexQueue);
	while (_queue.empty())
		_condQueue.wait(_mutexQueue);

	for (short i = 1; i <= CHUNK_SIZE; ++i)
		if (!_queue.empty())
		{
			Message* msg = _queue.front();
			_queue.pop_front();
			messages[*(msg->session()->socket())] = msg;
		}
		else
			break;
}
