
#ifndef HANDLER_CPP
#define HANDLER_CPP

#include <string>
#include <vector>
#include <map>
#include <Poco/Runnable.h>
#include <Poco/Mutex.h>
#include "Message.hpp"
#include "Server.hpp"


using std::string;
using std::vector;
using std::map;
using Poco::Runnable;
using Poco::FastMutex;


/**
Handles a client from the beginning until the end of session.
**/
class Handler : public Runnable
{
public:

	/**
	Chat line with 'from' address.
	**/
	struct From
	{
		string from;
		
		string chatLine;
		
		From() : from(), chatLine()
		{
		}
		
		From(string frm, string line) : from(frm), chatLine(line)
		{
		}
		
		bool empty()
		{
			if (from == "" && chatLine == "")
				return true;
			return false;
		}
	};

	Handler(Message* message);
	
	~Handler();

	void run();

	/**
	Username and password for each registered user.
	**/
	static map<string, string> _members;
	
private:

	static const short BUF_SIZE = 128;

	bool receive();

	bool send();

	void process();

	void removeUser(long sessionId);

	Message* _message;

	static FastMutex _mutexUsers;

	/**
	Maps session to username.
	**/
	static map<long, string> _users;

	static FastMutex _mutexInbox;
	
	/**
	Inbox with messages for a specific user sent by an user.
	**/
	static map<string, vector<From> > _inbox;
};


#endif // HANDLER_CPP
