
#include "Session.hpp"


Mutex Session::_mutexCounter;

long Session::_sessionCounter = 0;


Session::Session(StreamSocket* socket) : _socket(socket), _status(NONE)
{
	_id = Session::nextSessionCounter();
	time_t t;
	time(&t);
	_lastActivity = t;
}


Session::Session(long id, time_t lastActivity, StreamSocket* socket, Status status) :
	_id(id), _lastActivity(lastActivity), _socket(socket), _status(status)
{
}


Session::~Session()
{
	_socket->close();
	delete _socket;
	_id = 0;
	_lastActivity = 0;
}


long Session::id()
{
	return _id;
}


StreamSocket* Session::socket()
{
	return _socket;
}

	
void Session::socket(StreamSocket* s)
{
	_socket = s;
}


time_t Session::lastActivity()
{
	return _lastActivity;
}

	
void Session::lastActivityNow()
{
	time_t t;
	time(&t);
	_lastActivity = t;
}


Session::Status Session::status()
{
	return _status;
}

	
void Session::status(Session::Status s)
{
	_status = s;
}


long Session::nextSessionCounter()
{
	_mutexCounter.lock();
	long id = ++Session::_sessionCounter;
	_mutexCounter.unlock();
	return id;
}
