
#ifndef SESSION_HPP
#define SESSION_HPP

#include <Poco/Net/StreamSocket.h>
#include <Poco/Mutex.h>


using Poco::Net::StreamSocket;
using Poco::Mutex;


/**
Describes a client session (established via TCP).
It has unique ID, last activity time, corresponding socket.
**/
class Session
{
public:

	enum Status
	{
		NONE, // default
		NEW, // new conected client
		ACTIVE, // request/response generated
		DONE // processing finished on TCP and/or chat level
	};
	
	Session(StreamSocket* socket);

	Session(long sessionId, time_t lastActivity, StreamSocket* socket, Status status);
	
	~Session();
	
	long id();
	
	StreamSocket* socket();
	
	void socket(StreamSocket* s);
	
	time_t lastActivity();
	
	void lastActivityNow();
	
	Status status();
	
	void status(Status s);
	
private:

	static long nextSessionCounter();

	static Mutex _mutexCounter;
	
	static long _sessionCounter;
	
	long _id;

	time_t _lastActivity;

	StreamSocket* _socket;
	
	Status _status;
};


#endif // SESSION_HPP
