
#include <iostream>
#include <string>
#include <cstring>
#include <vector>
#include <utility>
#include <cstdlib>
#include <exception>
#include <cassert>
#include <Poco/Exception.h>
#include <Poco/Net/StreamSocket.h>
#include <Poco/Net/SocketAddress.h>
#include <Poco/ThreadPool.h>
#include <Poco/Thread.h>
#include <Poco/Runnable.h>
#include <Poco/Mutex.h>


using std::cout;
using std::endl;
using std::string;
using std::vector;
using std::pair;
using std::make_pair;
using std::exception;
using Poco::Exception;
using Poco::NoThreadAvailableException;
using Poco::Net::StreamSocket;
using Poco::Net::SocketAddress;
using Poco::ThreadPool;
using Poco::Thread;
using Poco::Runnable;
using Poco::Mutex;


/**
Makes random requests to the server.
**/
class Client : public Runnable
{
public:

	Client()
	{
		_commands.push_back("login");
		_commands.push_back("logout");
		_commands.push_back("send");
		_commands.push_back("receive");
		_commands.push_back("foobar");
		
		_users.push_back(make_pair("peter", "password"));
		_users.push_back(make_pair("pierre", "chiffre"));
		_users.push_back(make_pair("pera", "lozinka"));
		_users.push_back(make_pair("foo", "bar"));
		
		_chats.push_back("hello world");
		_chats.push_back("foo bar");
		_chats.push_back("yabadabadoo");
		_chats.push_back("opa bato");
		_chats.push_back("yeah");
		_chats.push_back("wow");
		_chats.push_back("bye");
	}

	void run()
	{
		StreamSocket client(SocketAddress("localhost:7000"));
		_mutex.lock();
		int userNo = rand() % _users.size();
		pair<string, string> userPass = _users[userNo];
		_mutex.unlock();
		string request = "command=login&username=" + userPass.first + "&password=" + userPass.second;
		int len = client.sendBytes(request.c_str(), request.length());
		assert(len == request.length());
		if (len == 0)
		{
			client.close();
			return;
		}
		
		const short RESPONSE_LEN = 128;
		char response[RESPONSE_LEN];
		::memset((void*)(response), '\0', RESPONSE_LEN);
		len = client.receiveBytes((void*)(response), RESPONSE_LEN - 2);
		if (len == 0)
		{
			client.close();
			return;
		}
		response[len] = '\0';
		
		unsigned int counter = 0;
		while (counter++ < 50)
		{
			_mutex.lock();
			int commandNo = rand() % _commands.size();
			string command = _commands[commandNo];
			int userNo = rand() % _users.size();
			pair<string, string> userPass = _users[userNo];
			_mutex.unlock();
			
			if (command == "send")
			{
				_mutex.lock();
				int userTo = rand() % _users.size();
				int chatNo = rand() % _chats.size();
				request = "command=send&to=" + _users[userTo].first + "&chat=" + _chats[chatNo] + "&from=" + userPass.first;
				_mutex.unlock();
			}
			else if (command == "receive")
				request = "command=receive";
			else if (command == "foobar")
				request = "command=foobar";

			
			int len = client.sendBytes(request.c_str(), request.length());
			assert(len == request.length());
			if (len == 0)
			{
				client.close();
				return;
			}
			
			const short RESPONSE_LEN = 128;
			char response[RESPONSE_LEN];
			::memset((void*)(response), '\0', RESPONSE_LEN);
			len = client.receiveBytes((void*)(response), RESPONSE_LEN - 2);
			if (len == 0)
			{
				client.close();
				return;
			}
			response[len] = '\0';
		}
		
		request = "command=logout";
		len = client.sendBytes(request.c_str(), request.length());
		assert(len == request.length());
		if (len == 0)
		{
			client.close();
			return;
		}
		
		::memset((void*)(response), '\0', RESPONSE_LEN);
		len = client.receiveBytes((void*)(response), RESPONSE_LEN - 2);
		if (len == 0)
		{
			client.close();
			return;
		}
		response[len] = '\0';
	}
	
private:

	Mutex _mutex;
	
	vector<string> _commands;
	
	vector<pair<string, string> > _users;
	
	vector<string> _chats;
};


int main()
{
	int counter = 0;
	while (counter++ < 50)
	{
		ThreadPool pool(1, 100);
		Client c;
		try
		{
			for (int j = 1; j <= 100; j++)
				pool.start(c);
			pool.joinAll();
		}
		catch (NoThreadAvailableException& exc)
		{
			cout << "main(): NoThreadAvailableException=" << exc.what() << ", pool.used()=" << pool.used() << endl;
		}
		catch (Exception& exc)
		{
			cout << "main(): Exception=" << exc.what() << ", pool.used()=" << pool.used() << endl;
		}

		cout << "main(): finishing: counter=" << counter << endl;
	}
	
	return EXIT_SUCCESS;
}
