/*
protocol:

client: command=login&username=petar&password=lozinka
server: command=login_ok
server: command=login_failed
server: command=login_notwice

client: command=logout
server: command=logout_ok

client: command=send&to=pierre&message=hello world
server: command=send_ok
server: command=send_no_user
server: command=send_not_authorized

client: command=receive
server: command=receive_ok&from=peter&message=hello world
server: command=receive_not_authorized
server: command=receive_no_messages

client: command=foobar
server: command=unknown command
*/


#include <iostream>
#include <stdexcept>
#include <string>
#include <Poco/ThreadPool.h>
#include <Poco/Logger.h>
#include <Poco/FormattingChannel.h>
#include <Poco/PatternFormatter.h>
#include <Poco/Message.h>
#include <Poco/FileChannel.h>
#include <Poco/LogStream.h>
#include "Server.hpp"
#include "Handler.hpp"


using std::string;
using std::cout;
using std::endl;
using std::exception;
using Poco::ThreadPool;
using Poco::Logger;
using Poco::FormattingChannel;
using Poco::PatternFormatter;
using Poco::FileChannel;
using Poco::LogStream;


int main()
{
	try
	{
		string logFileName = "server.log";
		FormattingChannel* channel = new FormattingChannel(new PatternFormatter("[%d-%m-%Y %H:%M:%S: %p]: %t"));
		channel->setChannel(new FileChannel(logFileName));
		channel->open();
		Logger& fileLogger = Logger::create("server", channel, Poco::Message::PRIO_INFORMATION);
		fileLogger.information("starting server");
		
		Handler::_members["peter"] = "password";
		Handler::_members["pierre"] = "chiffre";
		Handler::_members["pera"] = "lozinka";

		Dispatcher dispatcher;
		ThreadPool::defaultPool().start(dispatcher);
		Server server(&dispatcher);
		ThreadPool::defaultPool().start(server);
		while (true)
			;
		fileLogger.information("ending server");
	}
	catch (exception& exc)
	{
		cout << "main(): " << exc.what() << endl;
	}

	return EXIT_SUCCESS;
}
